@@ -540,7 +540,15 @@ class NoRootUserTest(fake_filesystem_unittest.TestCase):
             open(file_path, 'w')
 
 
-class PauseResumeTest(TestPyfakefsUnittestBase):
+class PauseResumeTest(fake_filesystem_unittest.TestCase):
+    def setUp(self):
+        self.real_temp_file = None
+        self.setUpPyfakefs()
+
+    def tearDown(self):
+        if self.real_temp_file is not None:
+            self.real_temp_file.close()
+
     def test_pause_resume(self):
         fake_temp_file = tempfile.NamedTemporaryFile()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
@@ -548,11 +556,11 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         self.pause()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
         self.assertFalse(os.path.exists(fake_temp_file.name))
-        real_temp_file = tempfile.NamedTemporaryFile()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.real_temp_file = tempfile.NamedTemporaryFile()
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         self.resume()
-        self.assertFalse(os.path.exists(real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_fs(self):
@@ -565,15 +573,15 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         self.fs.pause()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
         self.assertFalse(os.path.exists(fake_temp_file.name))
-        real_temp_file = tempfile.NamedTemporaryFile()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.real_temp_file = tempfile.NamedTemporaryFile()
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         # pause does nothing if already paused
         self.fs.pause()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         self.fs.resume()
-        self.assertFalse(os.path.exists(real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_contextmanager(self):
@@ -583,10 +591,10 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         with Pause(self):
             self.assertTrue(self.fs.exists(fake_temp_file.name))
             self.assertFalse(os.path.exists(fake_temp_file.name))
-            real_temp_file = tempfile.NamedTemporaryFile()
-            self.assertFalse(self.fs.exists(real_temp_file.name))
-            self.assertTrue(os.path.exists(real_temp_file.name))
-        self.assertFalse(os.path.exists(real_temp_file.name))
+            self.real_temp_file = tempfile.NamedTemporaryFile()
+            self.assertFalse(self.fs.exists(self.real_temp_file.name))
+            self.assertTrue(os.path.exists(self.real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_fs_contextmanager(self):
@@ -596,10 +604,10 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         with Pause(self.fs):
             self.assertTrue(self.fs.exists(fake_temp_file.name))
             self.assertFalse(os.path.exists(fake_temp_file.name))
-            real_temp_file = tempfile.NamedTemporaryFile()
-            self.assertFalse(self.fs.exists(real_temp_file.name))
-            self.assertTrue(os.path.exists(real_temp_file.name))
-        self.assertFalse(os.path.exists(real_temp_file.name))
+            self.real_temp_file = tempfile.NamedTemporaryFile()
+            self.assertFalse(self.fs.exists(self.real_temp_file.name))
+            self.assertTrue(os.path.exists(self.real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_without_patcher(self):
@@ -623,6 +631,7 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
             p.resume()
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
+        real_temp_file.close()
 
     def test_pause_resume_contextmanager(self):
         with Patcher() as p:
@@ -637,6 +646,7 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
                 self.assertTrue(os.path.exists(real_temp_file.name))
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
+        real_temp_file.close()
 
 
 class TestPyfakefsTestCase(unittest.TestCase):
